# tgEasy - Easy for a brighter Shine. A monkey pather add-on for Pyrogram
# Copyright (C) 2021 - 2022 Jayant Hegde Kageri <https://github.com/jayantkageri>

# This file is part of tgEasy.

# tgEasy is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# tgEasy is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.

# You should have received a copy of the GNU Lesser General Public License
# along with tgEasy.  If not, see <http://www.gnu.org/licenses/>.

import typing

import pyrogram

from tgEasy.scaffold import Scaffold

from ..helpers import handle_error


class Callback(Scaffold):
    def callback(
        self,
        data: typing.Union[str, list],
        self_admin: typing.Union[bool, bool] = False,
        filter: typing.Union[pyrogram.filters.Filter, pyrogram.filters.Filter] = None,
        *args,
        **kwargs,
    ):
        """
        ### `tgEasy.tgClient.callback`

        - A decorater to Register Callback Quiries in simple way and manage errors in that Function itself, alternative for `@pyrogram.Client.on_callback_query(pyrogram.filters.regex('^data.*'))`
        - Parameters:
        - data (str || list):
            - The callback query to be handled for a function

        - self_admin (bool) **optional**:
            - If True, the command will only executeed if the Bot is Admin in the Chat, By Default False

        - filter (`~pyrogram.filters`) **optional**:
            - Pyrogram Filters, hope you know about this, for Advaced usage. Use `and` for seaperating filters.

        #### Example
        .. code-block:: python
            import pyrogram
            from tgEasy import tgClient

            app = tgClient(pyrogram.Client())

            @app.command("start")
            async def start(client, message):
                await message.reply_text(
                f"Hello {message.from_user.mention}",
                reply_markup=pyrogram.types.InlineKeyboardMarkup([[
                    pyrogram.types.InlineKeyboardButton(
                    "Click Here",
                    "data"
                    )
                ]])
                )

            @app.callback("data")
            async def data(client, CallbackQuery):
            await CallbackQuery.answer("Hello :)", show_alert=True)
        """
        if filter:
            filter = pyrogram.filters.regex(f"^{data}.*") & args["filter"]
        else:
            filter = pyrogram.filters.regex(f"^{data}.*")

        def wrapper(func):
            async def decorator(client, CallbackQuery: pyrogram.types.CallbackQuery):
                if self_admin:
                    me = await client.get_chat_member(
                        CallbackQuery.message.chat.id, (await client.get_me()).id
                    )
                    if not me.status in (
                        pyrogram.enums.ChatMemberStatus.OWNER,
                        pyrogram.enums.ChatMemberStatus.ADMINISTRATOR,
                    ):
                        return await CallbackQuery.message.edit_text(
                            "I must be admin to execute this Command"
                        )
                    pass
                try:
                    await func(client, CallbackQuery)
                except pyrogram.errors.exceptions.forbidden_403.ChatAdminRequired:
                    pass
                except BaseException as e:
                    return await handle_error(e, CallbackQuery)

            self.__client__.add_handler(
                pyrogram.handlers.CallbackQueryHandler(decorator, filter)
            )
            return decorator

        return wrapper



# tgEasy - Easy for a brighter Shine. A monkey pather add-on for Pyrogram
# Copyright (C) 2021 - 2022 Jayant Hegde Kageri <https://github.com/jayantkageri>

# This file is part of tgEasy.

# tgEasy is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# tgEasy is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.

# You should have received a copy of the GNU Lesser General Public License
# along with tgEasy.  If not, see <http://www.gnu.org/licenses/>.

import typing

import pyrogram


async def check_rights(
    chat_id: typing.Union[int, int],
    user_id: typing.Union[int, int],
    rights: typing.Union[str, list],
    client,
) -> bool:
    """
    ### `tgEasy.check_rights`
    - Checks the Rights of an User
    - This is an Helper Function for `adminsOnly`

    - Parameters:
    - chat_id (int):
        - The Chat ID of Which Chat have to check the Rights.

    - user_id (int):
        - The User ID of Whose Rights have to Check.

    - rights (str):
        - The Rights have to Check.

    - client (`pyrogram.Client`):
        - From which Client to Check the Rights.

    - Returns:
    - `True` if the User have the Right.
    - `False` if the User don't have the Right.

    #### Example
    .. code-block:: python
        from tgEasy import tgClient, check_rights, get_user
        import pyrogram

        app = tgClient(pyrogram.Client())

        @app.command("ban", group_only=True, self_admin=True)
        async def ban(client, message):
        if not await check_rights(message.chat.id, message.from_user.id, "can_restrict_members"):
            return await message.reply_text("You don't have necessary rights to use this Command.")
        user = await get_user(message)
        await message.chat.kick_member(user.id)
    """
    try:
        user = await client.get_chat_member(chat_id, user_id)
    except:
        return False
    if user.status == "user":
        return False
    if user.status in (
        pyrogram.enums.ChatMemberStatus.OWNER,
        pyrogram.enums.ChatMemberStatus.ADMINISTRATOR,
    ):
        permission = []
        if user.privileges.can_manage_chat:
            permission.append("can_manage_chat")

        if user.privileges.can_delete_messages:
            permission.append("can_delete_messages")

        if user.privileges.can_manage_video_chats:
            permission.append("can_manage_video_chats")

        if user.privileges.can_restrict_members:
            permission.append("can_restrict_members")

        if user.privileges.can_promote_members:
            permission.append("can_promote_members")

        if user.privileges.can_change_info:
            permission.append("can_change_info")

        if user.privileges.can_post_messages:
            permission.append("can_post_messages")

        if user.privileges.can_edit_messages:
            permission.append("can_edit_messages")

        if user.privileges.can_invite_users:
            permission.append("can_invite_users")

        if user.privileges.can_pin_messages:
            permission.append("can_pin_messages")

        if user.privileges.is_anonymous:
            permission.append("is_anonymous")

        if isinstance(rights, str):
            if rights in permission:
                return True
            return False
        if isinstance(rights, list):
            for right in rights:
                if not right in permission:
                    return False
                return True
    return False


async def is_admin(
    chat_id: typing.Union[int, str], user_id: typing.Union[int, str], client
) -> bool:
    """
    ### `tgEasy.is_admin`
    - A Functions to Check if the User is Admin or not

    - Parameters:
        - chat_id (int):
            - The Chat ID of Which Chat have to check the Admin Status.

        - user_id (int):
            - The User ID of Whose Admin Status have to Check.

        - client (`pyrogram.Client`):
            - From which Client to Check the Admin Status.

    - Returns:
        - `True` if the User is Admin.
        - `False` if the User is't Admin.
    #### Example
    .. code-block:: python
        from tgEasy import tgClient, is_admin, adminsOnly
        import pyrogram

        app = tgClient(pyrogram.Client())

        @app.command("ban", group_only=True, self_admin=True)
        @app.adminsOnly("can_restrict_members")
        async def ban(client, message):
            if await is_admin(message.chat.id, (await get_user(mesasge)).id):
                return await message.reply_text("You can't Ban Admins.")
            await message.chat.kick_member((await get_user(message)).id)
            await message.reply_text("User has been Banned.")
    """
    try:
        user = await client.get_chat_member(chat_id, user_id)
    except:
        return False
    if user.status in (
        pyrogram.enums.ChatMemberStatus.OWNER,
        pyrogram.enums.ChatMemberStatus.ADMINISTRATOR,
    ):
        return True
    return False



# tgEasy - Easy for a brighter Shine. A monkey pather add-on for Pyrogram
# Copyright (C) 2021 - 2022 Jayant Hegde Kageri <https://github.com/jayantkageri>

# This file is part of tgEasy.

# tgEasy is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# tgEasy is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.

# You should have received a copy of the GNU Lesser General Public License
# along with tgEasy.  If not, see <http://www.gnu.org/licenses/>.

import os
import typing

import pyrogram

from ..config import Config


async def handle_error(
    error, m: typing.Union[pyrogram.types.Message, pyrogram.types.CallbackQuery]
):
    """
    ### `tgEasy.handle_error`
    - A Function to Handle the Errors in Functions.
    - This Sends the Error Log to the Log Group and Replies Sorry Message for the Users.
    - This is Helper for all of the functions for handling the Errors.

    - Parameters:
      - error:
        - The Exceptation.

      - m (`pyrogram.types.Message` or `pyrogram.types.CallbackQuery`):
        - The Message or Callback Query where the Error occurred.

    #### Exapmle
        .. code-block:: python
            from tgEasy import tgClient, handle_error
            import pyrogram

            app = tgClient(pyrogram.Client())

            @app.command("start")
            async def start(client, message):
            try:
                await message.reply_text("Hi :D') # I intentionally made an bug for Example :/
            except Exceptation as e:
                return await handle_error(e, message)
    """
    import traceback

    from .. import logging

    logging.exception(traceback.format_exc())

    with open("crash.log", "w+", encoding="utf-8") as log:
        log.write(traceback.format_exc())
        log.close()
    if isinstance(m, pyrogram.types.Message):
        try:
            await m.reply_text(
                "An Internal Error Occurred while Processing your Command, the Logs have been sent to the Owners of this Bot. Sorry for Inconvenience"
            )
            await m._client.send_document(
                Config.LOGS, "crash.log", caption="Crash Report of this Bot"
            )
        except:
            pass
    if isinstance(m, pyrogram.types.CallbackQuery):
        try:
            await m.message.delete()
            await m.message.reply_text(
                "An Internal Error Occurred while Processing your Command, the Logs have been sent to the Owners of this Bot. Sorry for Inconvenience"
            )
            await m.message._client.send_document(
                Config.LOGS, "crash.log", caption="Crash Report of this Bot"
            )
        except:
            pass
    os.remove("crash.log")
    return True



# tgEasy - Easy for a brighter Shine. A monkey pather add-on for Pyrogram
# Copyright (C) 2021 - 2022 Jayant Hegde Kageri <https://github.com/jayantkageri>

# This file is part of tgEasy.

# tgEasy is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# tgEasy is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.

# You should have received a copy of the GNU Lesser General Public License
# along with tgEasy.  If not, see <http://www.gnu.org/licenses/>.

import typing

import pyrogram


async def get_user(
    m: typing.Union[pyrogram.types.Message, pyrogram.types.CallbackQuery]
) -> pyrogram.types.User or bool:
    """
    ### `tgEasy.get_user`
    - Gets a User from Message/RepliedMessage/CallbackQuery
    - Parameters:
      - m (`~pyrogram.types.Message` || `~pyrogram.types.CallbackQuery`)
    - Returns:
      - `pyrogram.types.User` on Success
      - `False` on Error

    #### Example
        .. code-block:: python
            from tgEasy import get_user, command, adminsOnly

            @command("ban", group_only=True, self_admin=True)
            @adminsOnly("can_restrict_members")
            async def ban(client, message):
                user = await get_user(message)
                await message.chat.kick_member(user.id)
    """
    if isinstance(m, pyrogram.types.Message):
        message = m
        client = m._client
    if isinstance(m, pyrogram.types.CallbackQuery):
        message = m.message
        client = message._client
    if message.reply_to_message:
        if message.reply_to_message.sender_chat:
            return False
        return await client.get_users(message.reply_to_message.from_user.id)

    if len(message.command) > 1:
        command = message.command[1]
    else:
        command = None

    if command:
        if command.startswith("@") or command.isdigit():
            try:
                return await client.get_users(message.command[1])
            except pyrogram.errors.exceptions.bad_request_400.UsernameNotOccupied:
                pass
            except pyrogram.errors.exceptions.bad_request_400.UsernameInvalid:
                pass
            except pyrogram.errors.exceptions.bad_request_400.PeerIdInvalid:
                pass
            except IndexError:
                pass
    else:
        pass

    if message.entities:
        for mention in message.entities:
            if mention.type == "text_mention":
                user = mention.user.id
                break
        try:
            return await client.get_users(user)
        except:
            pass
    return False


async def get_user_adv(
    m: typing.Union[pyrogram.types.Message, pyrogram.types.CallbackQuery]
) -> pyrogram.types.User or bool:
    """
    ### `tgEasy.get_user_adv`
    - A Function to Get the User from the Message/CallbackQuery, If there is None arguments, returns the From User.
    - Parameters:
      - m (`pyrogram.types.Message` || `pyrogram.types.CallbackQuery`):
        - Message or Callbackquery.
    - Returns:
      - `pyrogram.types.User` on Success
      - `False` on Error

    #### Example
        .. code-block:: python
            from tgEasy import command, get_user_adv

            @command("id")
            async def id(client, message):
                user = await get_user_adv(message)
                await message.reply_text(f"Your ID is `{user.id}`")
    """
    if isinstance(m, pyrogram.types.Message):
        message = m
    if isinstance(m, pyrogram.types.CallbackQuery):
        message = m.message
    if message.sender_chat:
        return False
    try:
        if len(message.command) > 1:
            if message.command[1].startswith("@"):
                return await get_user(message)
            if message.command[1].isdigit():
                return await get_user(message)
            if "text_mention" in message.entities:
                return await get_user(message)
            if "from_user" in str(message.reply_to_message):
                return await get_user(message)
    except IndexError:
        pass
    except AttributeError:
        pass

    try:
        if "sender_chat" in str(message.reply_to_message):
            return False
        if "from_user" in str(message.reply_to_message):
            return await message._client.get_users(
                message.reply_to_message.from_user.id
            )
    except AttributeError:
        pass
    except Exception as e:
        pass

    return await message._client.get_users(message.from_user.id)



# tgEasy - Easy for a brighter Shine. A monkey pather add-on for Pyrogram
# Copyright (C) 2021 - 2022 Jayant Hegde Kageri <https://github.com/jayantkageri>

# This file is part of tgEasy.

# tgEasy is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# tgEasy is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.

# You should have received a copy of the GNU Lesser General Public License
# along with tgEasy.  If not, see <http://www.gnu.org/licenses/>.

import typing

import pyrogram
from cachetools import TTLCache

from tgEasy.scaffold import Scaffold

from ..helpers import check_rights, handle_error, is_admin

ANON = TTLCache(maxsize=250, ttl=30)


async def anonymous_admin(m: pyrogram.types.Message):
    """
    Helper function for Anonymous Admin Verification
    """
    keyboard = pyrogram.types.InlineKeyboardMarkup(
        [
            [
                pyrogram.types.InlineKeyboardButton(
                    text="Verify!",
                    callback_data=f"anon.{m.id}",
                ),
            ]
        ]
    )
    return await m.reply_text(
        "Click here to prove you are admin with the required rights to perform this action!",
        reply_markup=keyboard,
    )


class AdminsOnly(Scaffold):
    async def anonymous_admin_verification(
        client, CallbackQuery: pyrogram.types.CallbackQuery
    ):
        if int(
            f"{CallbackQuery.message.chat.id}{CallbackQuery.data.split('.')[1]}"
        ) not in set(ANON.keys()):
            try:
                await CallbackQuery.message.edit_text("Button has been Expired.")
            except pyrogram.types.RPCError:
                try:
                    await CallbackQuery.message.delete()
                except pyrogram.types.RPCError:
                    pass
            return
        cb = ANON.pop(
            int(f"{CallbackQuery.message.chat.id}{CallbackQuery.data.split('.')[1]}")
        )
        member = await CallbackQuery.message.chat.get_member(CallbackQuery.from_user.id)
        if bool(
            member.status
            not in (
                pyrogram.enums.ChatMemberStatus.OWNER,
                pyrogram.enums.ChatMemberStatus.ADMINISTRATOR,
            )
        ):
            return await CallbackQuery.answer("You need to be an admin to do this.")
        permission = cb[2]

        if isinstance(permission, str):
            if not await check_rights(
                CallbackQuery.message.chat.id,
                CallbackQuery.from_user.id,
                permission,
                client=client,
            ):
                return await CallbackQuery.message.edit_text(
                    f"You are Missing the following Rights to use this Command:\n{permission}",
                )
        if isinstance(permission, list):
            permissions = ""
            for perm in permission:
                if not await check_rights(
                    CallbackQuery.message.chat.id,
                    CallbackQuery.from_user.id,
                    perm,
                    client=client,
                ):
                    permissions += f"\n{perm}"
                    if not permissions == "":
                        return await CallbackQuery.message.edit_text(
                            f"You are Missing the following Rights to use this Command:{permissions}",
                        )
        try:
            await CallbackQuery.message.delete()
            await cb[1](client, cb[0])
        except pyrogram.errors.exceptions.forbidden_403.ChatAdminRequired:
            return await CallbackQuery.message.edit_text(
                "I must be admin to execute this Command",
            )
        except BaseException as e:
            return await handle_error(e, CallbackQuery)

    def adminsOnly(
        self,
        permission: typing.Union[str, list],
        TRUST_ANON_ADMINS: typing.Union[bool, bool] = False,
    ):
        """
        # `tgEasy.tgClient.adminsOnly`
        - A decorater for running the function only if the admin have the specified Rights.
        - If the admin is Anonymous Admin, it also checks his rights by making a Callback.
        - Parameters:
        - permission (str):
            - Permission which the User must have to use the Functions

        - TRUST_ANON_ADMIN (bool) **optional**:
            - If the user is an Anonymous Admin, then it bypasses his right check.

        # Example
        .. code-block:: python
            from tgEasy import tgClient
            import pyrogram

            app = tgClient(pyrogram.Client())

            @app.command("start")
            @app.adminsOnly("can_change_info")
            async def start(client, message):
                await message.reply_text(f"Hello Admin {message.from_user.mention}")
        """

        def wrapper(func):
            async def decorator(client, message):
                permissions = ""
                if not message.chat.type == pyrogram.enums.ChatType.SUPERGROUP:
                    return await message.reply_text(
                        "This command can be used in supergroups only.",
                    )
                if message.sender_chat and not TRUST_ANON_ADMINS:
                    ANON[int(f"{message.chat.id}{message.id}")] = (
                        message,
                        func,
                        permission,
                    )
                    return await anonymous_admin(message)
                if not await is_admin(
                    message.chat.id,
                    message.from_user.id,
                    client=client,
                ):
                    return await message.reply_text(
                        "Only admins can execute this Command!",
                    )
                if isinstance(permission, str):
                    if not await check_rights(
                        message.chat.id,
                        message.from_user.id,
                        permission,
                        client=client,
                    ):
                        return await message.reply_text(
                            f"You are Missing the following Rights to use this Command:\n{permission}",
                        )
                if isinstance(permission, list):
                    for perm in permission:
                        if not await check_rights(
                            message.chat.id,
                            message.from_user.id,
                            perm,
                            client=client,
                        ):
                            permissions += f"\n{perm}"
                    if not permissions == "":
                        return await message.reply_text(
                            f"You are Missing the following Rights to use this Command:{permissions}",
                        )
                try:
                    await func(client, message)
                except pyrogram.errors.exceptions.forbidden_403.ChatWriteForbidden:
                    await client.leave_chat(message.chat.id)
                except BaseException as exception:
                    await handle_error(exception, message)

            self.__client__.add_handler(
                pyrogram.handlers.CallbackQueryHandler(
                    AdminsOnly.anonymous_admin_verification,
                    pyrogram.filters.regex("^anon."),
                ),
            )
            return decorator

        return wrapper



# tgEasy - Easy for a brighter Shine. A monkey pather add-on for Pyrogram
# Copyright (C) 2021 - 2022 Jayant Hegde Kageri <https://github.com/jayantkageri>

# This file is part of tgEasy.

# tgEasy is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# tgEasy is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.

# You should have received a copy of the GNU Lesser General Public License
# along with tgEasy.  If not, see <http://www.gnu.org/licenses/>.

import typing

import pyrogram

from tgEasy.scaffold import Scaffold

from ..config import Config
from ..helpers import handle_error


class Command(Scaffold):
    def command(
        self,
        command: typing.Union[str, list],
        pm_only: typing.Union[bool, bool] = False,
        group_only: typing.Union[bool, bool] = False,
        self_admin: typing.Union[bool, bool] = False,
        self_only: typing.Union[bool, bool] = False,
        handler: typing.Optional[list] = Config.HANDLERS,
        filter: typing.Union[pyrogram.filters.Filter, pyrogram.filters.Filter] = None,
        *args,
        **kwargs
    ):
        """
        ### `tgEasy.tgClient.command`
        - A decorater to Register Commands in simple way and manage errors in that Function itself, alternative for `@pyrogram.Client.on_message(pyrogram.filters.command('command'))`
        - Parameters:
        - command (str || list):
            - The command to be handled for a function

        - group_only (bool) **optional**:
            - If True, the command will only executed in Groups only, By Default False.

        - pm_only (bool) **optional**:
            - If True, the command will only executed in Private Messages only, By Default False.

        - self_only (bool) **optional**:
            - If True, the command will only excute if used by Self only, By Default False.

        - handler (list) **optional**:
            - If set, the command will be handled by the specified Handler, By Default `Config.HANDLERS`.

        - self_admin (bool) **optional**:
            - If True, the command will only executeed if the Bot is Admin in the Chat, By Default False

        - filter (`~pyrogram.filters`) **optional**:
            - Pyrogram Filters, hope you know about this, for Advaced usage. Use `and` for seaperating filters.

        #### Example
        .. code-block:: python
            import pyrogram
            from tgEasy import tgClient

            app = tgClient(pyrogram.Client())

            @app.command("start", group_only=False, pm_only=False, self_admin=False, self_only=False, pyrogram.filters.chat("777000") and pyrogram.filters.text)
            async def start(client, message):
                await message.reply_text(f"Hello {message.from_user.mention}")
        """
        if filter:
            if self_only:
                filter = (
                    pyrogram.filters.command(command, prefixes=handler)
                    & filter
                    & pyrogram.filters.me
                )
            else:
                filter = (
                    pyrogram.filters.command(command, prefixes=handler)
                    & filter
                    & pyrogram.filters.me
                )
        else:
            if self_only:
                filter = (
                    pyrogram.filters.command(command, prefixes=handler)
                    & pyrogram.filters.me
                )
            else:
                filter = pyrogram.filters.command(command, prefixes=handler)

        def wrapper(func):
            async def decorator(client, message: pyrogram.types.Message):
                if (
                    self_admin
                    and message.chat.type != pyrogram.enums.ChatType.SUPERGROUP
                ):
                    return await message.reply_text(
                        "This command can be used in supergroups only."
                    )
                if self_admin:
                    me = await client.get_chat_member(
                        message.chat.id, (await client.get_me()).id
                    )
                    if not me.status in (
                        pyrogram.enums.ChatMemberStatus.OWNER,
                        pyrogram.enums.ChatMemberStatus.ADMINISTRATOR,
                    ):
                        return await message.reply_text(
                            "I must be admin to execute this Command"
                        )
                    pass
                if (
                    group_only
                    and message.chat.type != pyrogram.enums.ChatType.SUPERGROUP
                ):
                    return await message.reply_text(
                        "This command can be used in supergroups only."
                    )
                if pm_only and message.chat.type != pyrogram.enums.ChatType.PRIVATE:
                    return await message.reply_text(
                        "This command can be used in PMs only."
                    )
                try:
                    await func(client, message)
                except pyrogram.errors.exceptions.forbidden_403.ChatWriteForbidden:
                    await client.leave_chat(message.chat.id)
                except BaseException as exception:
                    return await handle_error(exception, message)

            self.__client__.add_handler(
                pyrogram.handlers.MessageHandler(callback=decorator, filters=filter)
            )
            return decorator

        return wrapper



# tgEasy - Easy for a brighter Shine. A monkey pather add-on for Pyrogram
# Copyright (C) 2021 - 2022 Jayant Hegde Kageri <https://github.com/jayantkageri>

# This file is part of tgEasy.

# tgEasy is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# tgEasy is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.

# You should have received a copy of the GNU Lesser General Public License
# along with tgEasy.  If not, see <http://www.gnu.org/licenses/>.

import typing

import pyrogram


async def send_typing(
    m: typing.Union[pyrogram.types.Message, pyrogram.types.CallbackQuery]
) -> bool:
    """
    ### `tgEasy.send_typing`
    - A Function to Send the Typing Status to the Chat.

    - Parameters:
      - m (`pyrogram.types.Message` || `pyrogram.types.CallbackQuery`):
        - Message or Callbackquery.

    #### Example
        .. code-block:: python
            from tgEasy import tgClinet, send_typing
            import pyrogram

            app = tgClient(pyrogram.Client())

            @app.command("start")
            async def start(client, message):
            await send_typing(message)
            await message.reply_text("Hello")
    """
    if isinstance(m, pyrogram.types.Message):
        message = m
    if isinstance(m, pyrogram.types.CallbackQuery):
        message = m.message
    for i in range(3):
        return await message._client.send_chat_action(message.chat.id, "typing")
